import tilebelt from "@mapbox/tilebelt";
import geojsonvt from "geojson-vt";
import vtpbf from "vt-pbf";

// [zoom, x, y] -> [x, y, zoom]
const tileToTilebeltTile = (
  tile: [zoom: number | string, x: number | string, y: number | string],
): [x: number, y: number, zoom: number] => [
  Number(tile[1]),
  Number(tile[2]),
  Number(tile[0]),
];

const calculateTolerance = (zoom: number) => {
  const initialTolerance = 0.1;
  return initialTolerance / Math.pow(2, zoom);
};

type Params = { layerId: string; tile: [zoom: string, x: string, y: string] };

// curl 'http://localhost:3000/api/cuzk/tiles/5/16/35391/22204'
// https://github.com/4xle/geojson-vt-stream
// https://github.com/mapbox/geojson-vt-cpp/pull/65
export async function GET(_: Request, { params }: { params: Params }) {
  const { layerId, tile } = params;
  const bbox = tilebelt.tileToBBOX(tileToTilebeltTile(tile));

  const tolerance = calculateTolerance(Number(tile[0]));

  const extent = {
    spatialReference: { latestWkid: 4326, wkid: 4326 },
    xmin: bbox[0],
    ymin: bbox[1],
    xmax: bbox[2],
    ymax: bbox[3],
  };

  const url = new URL(`https://ags.cuzk.cz`);
  url.pathname = `/arcgis/rest/services/RUIAN/Prohlizeci_sluzba_nad_daty_RUIAN/MapServer/${layerId}/query`;
  url.search = new URLSearchParams({
    f: "geojson",
    where: "1=1",
    outFields: "*",
    outSR: "4326",
    precision: "8",
    spatialRel: "esriSpatialRelIntersects",
    geometryType: "esriGeometryEnvelope",
    inSR: "4326",
    geometry: JSON.stringify(extent),
    quantizationParameters: JSON.stringify({
      extent,
      tolerance,
      mode: "view",
    }),
    resultType: "tile",
  }).toString();

  const geoJson = await fetch(url).then((response) => response.json());

  const tileIndex = geojsonvt(geoJson, {
    maxZoom: Number(tile[0]),
    tolerance: 0,
    extent: 4096,
    buffer: 64,
    debug: 0,
    lineMetrics: false,
    promoteId: null,
    generateId: false,
    indexMaxZoom: Number(tile[0]),
    indexMaxPoints: 0,
  });

  const tileData = tileIndex.getTile(...tile);

  if (tileData) {
    const tileBuffer = vtpbf.fromGeojsonVt({ geojsonLayer: tileData });
    return new Response(tileBuffer, {
      headers: {
        "Content-Type": "application/x-protobuf",
      },
    });
  } else {
    return new Response("No tile data found", { status: 404 });
  }
}

export const fetchCache = "default-no-store";
